/**
 * Easy Coverage for Java
 * <p>
 * Copyright 2011-2012 Tocea Technology Solutions http://www.tocea.com
 * <p>
 * This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
 * <p>
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 * <p>
 * You should have received a copy of the GNU Lesser General Public License along with this library. If not, see
 * <http://www.gnu.org/licenses/>.
 * <p>
 **/

package com.tocea.easycoverage.framework;



import java.lang.reflect.Modifier;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import junit.framework.TestSuite;

import org.easymock.EasyMock;
import org.slf4j.LoggerFactory;

import com.tocea.easycoverage.framework.api.IAutomaticTest;
import com.tocea.easycoverage.framework.api.IClassTest;
import com.tocea.easycoverage.framework.api.IObjectTest;



/**
 * This type describes the test harness.
 * <p>
 * 
 * @author $Author: sleroy $
 * @since 12 mai 2012
 * @copyright Copyright (C) 2010-2012 TOCEA
 */

public class TestHarness
{
    
    
    //
    // New logger.
    private static final org.slf4j.Logger LOGGER           = LoggerFactory.getLogger(TestHarness.class);
    
    
    private final List<IClassTest>        classBehaviours  = new ArrayList<IClassTest>(20);
    private final List<IObjectTest>       objectBehaviours = new ArrayList<IObjectTest>(20);
    
    private final List<SuiteClassTest>    classTests       = new ArrayList<SuiteClassTest>();
    private final List<SuiteInstanceTest> objectTests      = new ArrayList<SuiteInstanceTest>();
    
    private Set<Class<?>>                 classes          = new HashSet<Class<?>>(100);
    
    private Set<Object>                   objects          = new HashSet<Object>(100);
    
    private int                           failedInstances  = 0;
    
    
    private int                           failedClasses    = 0;
    
    
    
    /**
     * Builds a test harness.
     */
    public TestHarness() {
    
    
        super();
    }
    
    
    /**
     * This method performs the addition of a new behaviour to process on the class to test.
     * 
     * @param _newInstance
     *            the new instance.
     */
    public void addBehaviour(final IAutomaticTest _newInstance) {
    
    
        if (_newInstance == null) {
            throw new IllegalArgumentException();
        }
        if (_newInstance instanceof IClassTest) {
            this.classBehaviours.add((IClassTest) _newInstance);
        }
        if (_newInstance instanceof IObjectTest) {
            this.objectBehaviours.add((IObjectTest) _newInstance);
        }
    }
    
    
    /**
     * This method adds a new class test.
     * 
     * @param _test
     *            the test
     */
    public void addClassTest(final SuiteClassTest _test) {
    
    
        this.classTests.add(_test);
    }
    
    
    /**
     * This method adds an object test.
     * 
     * @param _test
     *            the test
     */
    public void addObjectTest(final SuiteInstanceTest _test) {
    
    
        this.objectTests.add(_test);
    }
    
    
    /**
     * This method performs the creation of instances from the classes
     */
    public void createInstancesFromClasses() {
    
    
        for (final Class classTest : this.classes) {
            final int modifiers = classTest.getModifiers();
            if (Modifier.isAbstract(modifiers) || Modifier.isInterface(modifiers) || classTest.isEnum()) {
                continue;
            }
            try {
                this.objects.add(EasyMock.createMock(classTest));
            } catch (final IllegalArgumentException _e) {
                try {
                    this.objects.add(classTest.newInstance());
                } catch (final Exception e) {
                    LOGGER.error("Cannot instantiate {}", classTest, e);
                    
                }
            }
            
        }
    }
    
    
    /**
     * Returns the classTests field.
     * 
     * @return the classTests
     */
    public List<SuiteClassTest> getClassTests() {
    
    
        return this.classTests;
    }
    
    
    /**
     * Returns the objectTests field.
     * 
     * @return the objectTests
     */
    public List<SuiteInstanceTest> getObjectTests() {
    
    
        return this.objectTests;
    }
    
    
    /**
     * This method performs the processing of all tests.
     * 
     * @return the processing of all tests.
     */
    public boolean processTests() {
    
    
        boolean res = true;
        LOGGER.debug("Initialization of the tests"); //$NON-NLS-1$
        final List<SuiteInstanceTest> testingInstances = this.testingInstances();
        final List<SuiteClassTest> testingClasses = this.testingClasses();
        
        for (final SuiteInstanceTest suite : testingInstances) {
            final boolean individualRes = suite.run().wasSuccessful();
            if (!individualRes) {
                LOGGER.error("\t ->[FAILED]"); //$NON-NLS-1$
                this.failedInstances++;
            }
            res &= individualRes;
            
        }
        for (final SuiteClassTest suite : testingClasses) {
            final boolean individualRes = suite.run().wasSuccessful();
            if (!individualRes) {
                LOGGER.error("\t ->[FAILED]"); //$NON-NLS-1$
                this.failedClasses++;
            }
            res &= individualRes;
        }
        
        LOGGER.debug("Tests finished {} {}", this.failedInstances, this.failedClasses); //$NON-NLS-1$
        LOGGER.debug("\tInstance failed {} on {}", this.failedInstances, this.objects.size()); //$NON-NLS-1$
        LOGGER.debug("\tClasses failed {} on {}", this.failedClasses, this.classes.size()); //$NON-NLS-1$
        return res;
        
    }
    
    
    /**
     * This method provides a bank of tests for junit
     * 
     * @param _suite
     *            the suite
     */
    public void provideTests(final TestSuite _suite) {
    
    
        for (final SuiteClassTest test : this.testingClasses()) {
            _suite.addTest(test);
        }
        for (final SuiteInstanceTest test : this.testingInstances()) {
            _suite.addTest(test);
        }
    }
    
    
    /**
     * This method sets the classes to analysis.
     * 
     * @param _setClasses
     *            the classes to analysis.
     */
    public void setClasses(final Set<Class<?>> _setClasses) {
    
    
        this.classes = _setClasses;
    }
    
    
    /**
     * This method sets the classes to analysis.
     * 
     * @param _setObjects
     *            the classes to analysis.
     */
    public void setObjects(final Set<Object> _setObjects) {
    
    
        this.objects = _setObjects;
        
        
    }
    
    
    /**
     * This method performs the testing classes.
     * 
     * @return the result of tests
     */
    private List<SuiteClassTest> testingClasses() {
    
    
        final List<SuiteClassTest> res = new ArrayList<SuiteClassTest>(this.classes.size());
        LOGGER.debug("{} classes to test; ", this.classes.size()); //$NON-NLS-1$
        for (final Class<?> clazz : this.classes) {
            res.add(new SuiteClassTest(clazz, this.classBehaviours));
        }
        return res;
        
    }
    
    
    /**
     * This method performs the testing classes.
     * 
     * @return the result of tests
     */
    private List<SuiteInstanceTest> testingInstances() {
    
    
        final List<SuiteInstanceTest> res = new ArrayList<SuiteInstanceTest>(this.objects.size());
        LOGGER.debug("{} instances to test; ", this.objects.size()); //$NON-NLS-1$
        for (final Object object : this.objects) {
            res.add(new SuiteInstanceTest(object, this.objectBehaviours));
        }
        return res;
    }
}
